/*
 * Copyright 2012 Hanson Robokind LLC.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.robokind.impl.speech.osgi;

import java.util.Map;
import java.util.Properties;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.jms.Connection;
import javax.jms.Destination;
import javax.jms.JMSException;
import javax.jms.Session;
import org.robokind.api.common.osgi.AbstractLifecycleProvider;
import org.robokind.api.common.osgi.DescriptorListBuilder;
import org.robokind.api.common.osgi.ServiceLifecycleProvider;
import org.robokind.api.speech.SpeechService;
import org.robokind.impl.messaging.utils.ConnectionManager;
import org.robokind.impl.speech.JMSSpeechService;

/**
 *
 * @author Matthew Stevenson <www.robokind.org>
 */
public class JMSRemoteSpeechServiceLifecycleProvider extends 
        AbstractLifecycleProvider<SpeechService, JMSSpeechService> {
    private final static Logger theLogger = 
            Logger.getLogger(ServiceLifecycleProvider.class.getName());
    private final static String theDepConnectionId = "motionConnection";
    private final static String theDepCommandDestId = "commandDestination";
    private final static String theDepErrorDestId = "errorDestination";
    private final static String theDepRequestDestId = "requestDestination";
    private final static String theDepEventDestId = "responseDestination";
    
    private String mySpeechServiceId;
          
    public JMSRemoteSpeechServiceLifecycleProvider(
            String speechServiceId, String connId, 
            String cmdDestId, String errDestId, 
            String reqDestId, String respDestId){
        super(new DescriptorListBuilder()
                .addId(theDepConnectionId, Connection.class, 
                        ConnectionManager.PROP_CONNECTION_ID, connId)
                .addId(theDepRequestDestId, Destination.class, 
                        ConnectionManager.PROP_DESTINATION_ID, reqDestId)
                .addId(theDepEventDestId, Destination.class, 
                        ConnectionManager.PROP_DESTINATION_ID, respDestId)
                .addId(theDepCommandDestId, Destination.class, 
                        ConnectionManager.PROP_DESTINATION_ID, cmdDestId)
                .addId(theDepErrorDestId, Destination.class, 
                        ConnectionManager.PROP_DESTINATION_ID, errDestId)
                .getDescriptors());
        if(speechServiceId == null){
            throw new NullPointerException();
        }
        mySpeechServiceId = speechServiceId;
        myRegistrationProperties = new Properties();
        myRegistrationProperties.put(SpeechService.PROP_ID, speechServiceId);
    }

    @Override
    protected JMSSpeechService create(Map<String, Object> services) {
        Connection con = (Connection)services.get(theDepConnectionId);
        Destination cmdDest = (Destination)services.get(theDepCommandDestId);
        Destination errDest = (Destination)services.get(theDepErrorDestId);
        Destination reqDest = (Destination)services.get(theDepRequestDestId);
        Destination respDest = (Destination)services.get(theDepEventDestId);
        Session session;
        try{
            session = con.createSession(false, Session.CLIENT_ACKNOWLEDGE);
        }catch(JMSException ex){
            theLogger.log(Level.WARNING, "Unable to create Session.", ex);
            return null;
        }
        try{
            JMSSpeechService service = new JMSSpeechService(mySpeechServiceId, 
                    session, cmdDest, errDest, reqDest, respDest);
            service.start();
            return service;
        }catch(Exception ex){
            theLogger.log(Level.WARNING, 
                    "There was an error creating and "
                    + "starting the JMSSpeechService.", ex);
            return null;
        }
    }

    @Override
    protected void handleChange(String serviceId, Object service) {
    }

    @Override
    public Class<SpeechService> getServiceClass() {
        return SpeechService.class;
    }
    
}
